# Copyright 2017 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the &quot;License&quot;);
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an &quot;AS IS&quot; BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# NOTE: This class is auto generated by the swagger code generator program.
# https://github.com/swagger-api/swagger-codegen.git
# Do not edit the class manually.

defmodule GoogleApi.AdSenseHost.V41.Api.Associationsessions do
  @moduledoc """
  API calls for all endpoints tagged `Associationsessions`.
  """

  alias GoogleApi.AdSenseHost.V41.Connection
  import GoogleApi.AdSenseHost.V41.RequestBuilder


  @doc """
  Create an association session for initiating an association with an AdSense user.

  ## Parameters

  - connection (GoogleApi.AdSenseHost.V41.Connection): Connection to server
  - product_code (List[String]): Products to associate with the user.
  - website_url (String): The URL of the user&#39;s hosted website.
  - opts (KeywordList): [optional] Optional parameters
    - :alt (String): Data format for the response.
    - :fields (String): Selector specifying which fields to include in a partial response.
    - :key (String): API key. Your API key identifies your project and provides you with API access, quota, and reports. Required unless you provide an OAuth 2.0 token.
    - :oauth_token (String): OAuth 2.0 token for the current user.
    - :pretty_print (Boolean): Returns response with indentations and line breaks.
    - :quota_user (String): Available to use for quota purposes for server-side applications. Can be any arbitrary string assigned to a user, but should not exceed 40 characters. Overrides userIp if both are provided.
    - :user_ip (String): IP address of the site where the request originates. Use this if you want to enforce per-user limits.
    - :user_locale (String): The preferred locale of the user.
    - :website_locale (String): The locale of the user&#39;s hosted website.

  ## Returns

  {:ok, %GoogleApi.AdSenseHost.V41.Model.AssociationSession{}} on success
  {:error, info} on failure
  """
  @spec adsensehost_associationsessions_start(Tesla.Env.client, list(String.t), String.t, keyword()) :: {:ok, GoogleApi.AdSenseHost.V41.Model.AssociationSession.t} | {:error, Tesla.Env.t}
  def adsensehost_associationsessions_start(connection, product_code, website_url, opts \\ []) do
    optional_params = %{
      :"alt" => :query,
      :"fields" => :query,
      :"key" => :query,
      :"oauth_token" => :query,
      :"prettyPrint" => :query,
      :"quotaUser" => :query,
      :"userIp" => :query,
      :"userLocale" => :query,
      :"websiteLocale" => :query
    }
    %{}
    |> method(:get)
    |> url("/associationsessions/start")
    |> add_param(:query, :"productCode", product_code)
    |> add_param(:query, :"websiteUrl", website_url)
    |> add_optional_params(optional_params, opts)
    |> Enum.into([])
    |> (&Connection.request(connection, &1)).()
    |> decode(%GoogleApi.AdSenseHost.V41.Model.AssociationSession{})
  end

  @doc """
  Verify an association session after the association callback returns from AdSense signup.

  ## Parameters

  - connection (GoogleApi.AdSenseHost.V41.Connection): Connection to server
  - token (String): The token returned to the association callback URL.
  - opts (KeywordList): [optional] Optional parameters
    - :alt (String): Data format for the response.
    - :fields (String): Selector specifying which fields to include in a partial response.
    - :key (String): API key. Your API key identifies your project and provides you with API access, quota, and reports. Required unless you provide an OAuth 2.0 token.
    - :oauth_token (String): OAuth 2.0 token for the current user.
    - :pretty_print (Boolean): Returns response with indentations and line breaks.
    - :quota_user (String): Available to use for quota purposes for server-side applications. Can be any arbitrary string assigned to a user, but should not exceed 40 characters. Overrides userIp if both are provided.
    - :user_ip (String): IP address of the site where the request originates. Use this if you want to enforce per-user limits.

  ## Returns

  {:ok, %GoogleApi.AdSenseHost.V41.Model.AssociationSession{}} on success
  {:error, info} on failure
  """
  @spec adsensehost_associationsessions_verify(Tesla.Env.client, String.t, keyword()) :: {:ok, GoogleApi.AdSenseHost.V41.Model.AssociationSession.t} | {:error, Tesla.Env.t}
  def adsensehost_associationsessions_verify(connection, token, opts \\ []) do
    optional_params = %{
      :"alt" => :query,
      :"fields" => :query,
      :"key" => :query,
      :"oauth_token" => :query,
      :"prettyPrint" => :query,
      :"quotaUser" => :query,
      :"userIp" => :query
    }
    %{}
    |> method(:get)
    |> url("/associationsessions/verify")
    |> add_param(:query, :"token", token)
    |> add_optional_params(optional_params, opts)
    |> Enum.into([])
    |> (&Connection.request(connection, &1)).()
    |> decode(%GoogleApi.AdSenseHost.V41.Model.AssociationSession{})
  end
end
